import { __awaiter } from "tslib";
import { MessageType, } from '@genesislcap/foundation-comms';
import { logger } from '../utils';
import { BaseServerSideDatasource } from './server-side.resource-base';
/**
 * The REQUEST_SERVER IServerSideDatasource implementation, used for SSRM (Server-Side Row Model) in the grid.
 * @alpha
 */
export class ReqRepServerSideDatasource extends BaseServerSideDatasource {
    constructor(options) {
        var _a;
        super(options);
        this.createReqRepRequestFunc = options.createReqRepRequestFunc;
        if (!((_a = options.resourceParams) === null || _a === void 0 ? void 0 : _a.VIEW_NUMBER)) {
            this.resourceParams.VIEW_NUMBER = this.zeroBasedViewNumber ? 0 : 1;
        }
    }
    getRows(params) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.setupFilteringAndSorting(params);
            if (this.pagination && !this.isNewPageSize && this.currentSequenceId > 0) {
                params.success(this.lastSuccessRowData);
                return;
            }
            if (this.currentSequenceId !== null && this.currentSequenceId >= 0 && this.moreRows) {
                if (this.pagination) {
                    this.resourceParams.VIEW_NUMBER = this.getPageNumber(params.request.startRow, this.maxRows, this.maxView);
                }
                else {
                    this.resourceParams.VIEW_NUMBER += 1;
                }
            }
            const requestParams = {
                DETAILS: {
                    CRITERIA_MATCH: this.resourceParams.CRITERIA_MATCH,
                    MAX_ROWS: this.resourceParams.MAX_ROWS,
                    VIEW_NUMBER: this.resourceParams.VIEW_NUMBER,
                },
            };
            const requestResult = yield this.createReqRepRequestFunc(requestParams);
            // TODO: massive limitation on the GSF. It should return -1 if it's already the last view (but it doesn't, next will have 0 rows then the NEXT_VIEW=-1).
            let isLastView = false;
            if (requestResult.NEXT_VIEW !== -1) {
                const preCheckRequestParams = Object.assign(Object.assign({}, requestParams), { DETAILS: Object.assign(Object.assign({}, requestParams.DETAILS), { VIEW_NUMBER: requestResult.NEXT_VIEW }) });
                const moreRowsPreCheck = yield this.createReqRepRequestFunc(preCheckRequestParams);
                isLastView = moreRowsPreCheck.NEXT_VIEW === -1;
            }
            let applyResult = true;
            if (applyResult) {
                this.applyServerSideData(params, requestResult, isLastView);
                applyResult = false;
            }
        });
    }
    applyServerSideData(params, result, isLastView) {
        var _a;
        const messageType = result.MESSAGE_TYPE;
        if (messageType &&
            (messageType === MessageType.LOGOFF_ACK || messageType === MessageType.MSG_NACK)) {
            params.fail();
            return;
        }
        this.moreRows =
            !isLastView && result.NEXT_VIEW !== -1 && result.NEXT_VIEW > this.resourceParams.VIEW_NUMBER;
        if (result.REPLY) {
            this.handleCurrentStreamLoad(result.REPLY);
        }
        else {
            this.rowData = new Map();
            const successRowData = { rowData: [] };
            successRowData.rowCount = 0;
            params.success(successRowData);
            return;
        }
        this.currentSequenceId = this.resourceParams.VIEW_NUMBER; // result.NEXT_VIEW > 0 ? result.NEXT_VIEW - 1 : 0;
        this.sourceRef = result.SOURCE_REF;
        this.serverRowsCount += (_a = result.ROWS_COUNT) !== null && _a !== void 0 ? _a : this.rowData.size;
        const successRowData = {
            rowData: Array.from(this.rowData.values()),
        };
        successRowData.rowCount = this.getCorrectRowCount(params);
        this.lastSuccessRowData = successRowData;
        params.success(successRowData);
    }
    getCorrectRowCount(params) {
        const lastRow = this.moreRows ? -1 : params.request.startRow + this.serverRowsCount;
        const currentLastRowNumber = params.request.endRow;
        this.calculatedRowsCount = this.serverRowsCount || currentLastRowNumber;
        const defaultCount = Math.min(lastRow, this.maxView);
        let rowCount;
        if (this.pagination) {
            rowCount = !this.resourceParams.CRITERIA_MATCH ? Math.min(lastRow, this.maxView) : undefined;
            if (this.resourceParams.CRITERIA_MATCH) {
                rowCount = defaultCount;
            }
            if (!this.moreRows && lastRow > this.calculatedRowsCount) {
                rowCount = this.calculatedRowsCount;
            }
            return rowCount;
        }
        if (currentLastRowNumber === lastRow) {
            rowCount = lastRow;
        }
        else if (currentLastRowNumber > this.maxView ||
            currentLastRowNumber > this.calculatedRowsCount) {
            rowCount = Math.min(defaultCount, this.calculatedRowsCount);
        }
        else if (!this.moreRows && lastRow > this.calculatedRowsCount) {
            rowCount = this.calculatedRowsCount;
        }
        return rowCount;
    }
    handleCurrentStreamLoad(reply) {
        if (!reply || reply.length == 0)
            return;
        const rows = new Map();
        reply.forEach((insertData) => {
            const insertDataRowId = insertData[this.rowId];
            if (insertDataRowId) {
                rows.set(insertDataRowId, insertData);
                return;
            }
            const noRowIdMessage = `The specified rowId (${this.rowId}) is not present in the data. Since this is a snapshot (isSnapshot=true), an auto-generated UUID will be used. For custom identifiers, use the 'row-id' attribute.`;
            logger.error(noRowIdMessage);
            const tempId = this.uuid.createId();
            insertData[this.rowId] = tempId;
            rows.set(tempId, insertData);
        });
        this.rowData = rows;
    }
}
